@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2019 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2021 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -849,6 +849,64 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
                  both = FALSE, diag = TRUE) {
 
 
+
+  jay <- j
+  kay <- k
+
+  if (M == 1)
+    if (!diag) stop("cannot handle M == 1 and diag = FALSE")
+
+  if (M == 1) {
+    if (both) return(list(row.index = 1, col.index = 1)) else return(1)
+  }
+     
+  upper <- if (diag) M else M - 1
+  i2 <- as.list(upper:1)
+  i2 <- lapply(i2, seq)
+  i2 <- unlist(i2)
+
+  i1 <- matrix(1:M, M, M)
+  i1 <- if (diag) c(i1[row(i1) >= col(i1)]) else
+                  c(i1[row(i1) >  col(i1)])
+
+  if (both) {
+    list(row.index = i2, col.index = i1)
+  } else {
+    if (any(jay > M) || any(kay > M) || any(jay < 1) || any(kay < 1))
+      stop("range error in argument 'j' or 'k'")
+
+    if (length(jay) < length(kay)) jay <- rep_len(jay, length(kay))
+    if (length(kay) < length(jay)) kay <- rep_len(kay, length(jay))
+
+    if (all(c(length(jay), length(kay)) == 1)) {
+      both <- (i1 == jay & i2 == kay) | (i1 == kay & i2 == jay)
+      return((seq_along(i2))[both])
+    } else {
+    vector.big <- 10000 * pmin(i1, i2) + pmax(i1, i2)
+    vector.sml <- 10000 * pmin(jay, kay) + pmax(jay, kay)
+      match.sml <- match(vector.sml, vector.big)
+      match.sml
+    }
+  }
+}  # iam
+
+
+
+
+
+
+
+if (FALSE)
+ miam <- function(j, k, M1,  # M1 used to be called M
+                  NOS = 1,  # This argument is new
+                  both = FALSE, diag = TRUE) {
+
+  if (NOS == 1)
+    return(iam(j = j, k = k, M = M1, both = both, diag = diag))
+
+  M <- M1 * NOS
+
+
   jay <- j
   kay <- k
 
@@ -856,7 +914,8 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
     if (!diag) stop("cannot handle this")
 
   if (M == 1)
-    if (both) return(list(row.index = 1, col.index = 1)) else return(1)
+    if (both) return(list(row.index = 1, col.index = 1)) else
+    return(1)
 
   upper <- if (diag) M else M - 1
   i2 <- as.list(upper:1)
@@ -876,7 +935,7 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
             (i1 == kay & i2 == jay)
     (seq_along(i2))[both]
   }
-}  # iam
+}  # miam
 
 
 
@@ -935,33 +994,31 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
- a2m <- function(a, hbw = M) {
-
-
+ a2m <- function(a, trim = FALSE) {
 
+     
   if (is.matrix(a) && ncol(a) == nrow(a))
     a <- array(a, c(nrow(a), ncol(a), 1))
   if (!is.array(a))
-    dim(a) <- c(1,1,length(a))
+    dim(a) <- c(1, 1, length(a))
 
   M <- dim(a)[1]
+  if (M != dim(a)[2])
+    stop("argument 'a' does not contain square matrices")
   n <- dim(a)[3]
-  dimm.value <- dimm(M, hbw)
+  dimm.value <- dimm(M)
   index <- iam(NA, NA, M, both = TRUE, diag = TRUE)
 
+  mat <- matrix(0, n, dimm.value)
+  for (jay in seq(dimm.value))
+    mat[, jay] <- a[index$row[jay], index$col[jay], ]
 
-  fred <- .C("a2mccc",
-             as.double(a), m = double(dimm.value * n),
-      as.integer(dimm.value),
-      as.integer(index$row-1),  as.integer(index$col-1),
-      as.integer(n),  as.integer(M), NAOK = TRUE)
-  dim(fred$m) <- c(dimm.value,n)
-  fred$m <- t(fred$m)
+  if (trim)
+    for (jay in dimm.value:1) {
+      if (all(mat[, jay] == 0)) mat <- mat[, -jay] else break
+    }
 
-  if (hbw != M)
-    attr(fred$m, "hbw") <- hbw
-  if (length(lpn <- dimnames(a)[[1]]) != 0)
-    attr(fred$m, "predictors.names") <- lpn
+  mat
 }  # a2m
 
 
@@ -1006,7 +1063,6 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
 
 
 
-
   if (length(wz <- object@weights) && !ignore.slot && !deriv.arg) {
     return(wz)
   }
@@ -1043,6 +1099,9 @@ cm.nointercept.VGAM <- function(constraints, x, nointercept, M) {
   if (!length(y))
     y <- depvar(object)
 
+    offset <- object@offset  # Could be cbind(0)
+    if (all(dim(offset) == 1)) offset <- c(offset)  # 0 (not a matrix)
+
 
   X.vlm.save <- model.matrixvlm(object, type = "vlm")
 
@@ -1654,7 +1713,8 @@ w.y.check <- function(w, y,
          "at least ", ncol.y.max, " needed")
 
   if (min(w) <= 0)
-    stop("prior-weight variable 'w' must contain positive values only")
+    stop("prior-weight variable 'w' must contain positive ",
+         "values only")
 
   if (is.numeric(colsyperw) && ncol(y) %% colsyperw != 0)
     stop("number of columns of the response variable 'y' is not ",
@@ -1741,7 +1801,7 @@ arwz2wz <- function(arwz, M = 1, M1 = 1, rm.trailing.cols = TRUE,
 
 
 
-wz.merge <- function(wz1, wz2, M1, M2, rm.trailing.cols = TRUE) {
+ wz.merge <- function(wz1, wz2, M1, M2, rm.trailing.cols = TRUE) {
 
 
   if (!is.matrix(wz1))
@@ -1863,8 +1923,21 @@ bisection.basic <-
 
 
 
+
+      
+
+  if (any(is.infinite(b))) {
+    warning("replacing 'b' values of Inf by a large value")
+    b[is.infinite(b)] <- .Machine$double.xmax / 4
+  }
+
+ 
+
+
   if (is.null(nmax)) {
     nmax <- round(log2(max(b - a)) - log2(min(tol))) + 4
+    if (!is.finite(nmax))
+      nmax <- log2(.Machine$double.xmax) - 5
   }
   signtest <- (sign(f(a, ...)) * sign(f(b, ...)) <= 0)
 
@@ -1973,5 +2046,100 @@ attr.assign.x.vglm <- function(object) {
 
 
 
+muxtXAX <- function(A, X, M) {
+
+  if ((n <- nrow(X)) != nrow(A))
+    stop("arguments 'X' and 'A' are not conformable")
+
+  ans1 <- array(0, c(n, M, M))
+  for (eye in 1:M)
+    for (jay in 1:M)
+      for (kay in 1:M)
+        ans1[, eye, jay] <- ans1[, eye, jay] +
+          A[, iam(eye, kay, M)] * X[, iam(kay, jay, M)]
+
+  ans2 <- matrix(0, n, dimm(M))
+  for (eye in 1:M)
+    for (jay in eye:M)
+      for (kay in 1:M)
+        ans2[, iam(eye, jay, M)] <- ans2[, iam(eye, jay, M)] +
+        X[, iam(kay, eye, M)] * ans1[, kay, jay]
+  ans2
+}  # muxtXAX
+
+
+
+
+
+
+
+
+
+
+
+ trim.constraints <-
+   function(object, sig.level = 0.05,
+            max.num = Inf, ...) {
+
+
+
+
+
+  if (!is(object, "vlm"))
+    stop("argument 'object' should inherit from the 'vglm' class")
+
+
+  if (!is.finite(sig.level) || !is.numeric(sig.level) ||
+      min(sig.level) < 0 || max(sig.level) > 1)
+    stop("bad input for argument 'sig.level'")
+
+  csfit <- coef(summary(object, HDEtest = FALSE))
+  X.small <- model.matrix(object, type = "lm")
+  X.assign <- attr(X.small, "assign")
+  cmlist2 <-
+  cmlist1 <- constraints(object, type = "term")
+  ncolHlist <- sapply(cmlist1, ncol)
+  colsperterm <- sapply(X.assign, length)
+  endpts <- cumsum(colsperterm * ncolHlist)
+  sig.level <- rep_len(sig.level, max(endpts))
+
+
+  pvs.vec <- csfit[, "Pr(>|z|)"]
+  if (any(!is.finite(pvs.vec)))
+    stop("cannot handle any NAs, etc. as p-values")
+  if (is.finite(max.num)) {
+    if (!is.Numeric(max.num, positive = TRUE, integer.valued = TRUE))
+      stop("bad input for argument 'max.num'")
+    spvs.vec <- sort(pvs.vec, decreasing = TRUE)
+    one.more <- as.numeric(max.num < length(spvs.vec))
+    use.sig.level <- mean(spvs.vec[max.num + (0:one.more)])
+    sig.level[sig.level < use.sig.level] <- use.sig.level
+  }
+
+  for (kay in rev(seq(length(cmlist1)))) {
+    cm.kay0 <- cm.kay <- cmlist1[[kay]]
+    for (cay in rev(seq(ncol(cm.kay0)))) {
+      cptr <- (if (kay == 1) 0 else endpts[kay - 1]) +
+              (ncolHlist[kay]) * seq(colsperterm[kay]) +
+              cay - ncol(cm.kay0)
+      if (all(pvs.vec[cptr] > sig.level[cptr]))
+        cm.kay <- cm.kay[, -cay, drop = FALSE]
+    }  # cay
+    cmlist2[[kay]] <- if (length(cm.kay)) cm.kay else NULL
+  }
+
+
+
+
+  cmlist2
+}  # trim.constraints
+
+
+
+
+
+
+
+
 
 
