@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2021 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -13,6 +13,8 @@
 
 
 
+
+
 N.hat.posbernoulli <-
   function(eta, link, earg = list(),
            R = NULL, w = NULL,
@@ -118,7 +120,9 @@ N.hat.posbernoulli <-
                     c(sqrt(ss2 + t(dvect) %*% covun %*% dvect)) else
                     c(sqrt(ss2))
       )
-}
+}  # N.hat.posbernoulli
+
+
 
 
 
@@ -144,7 +148,7 @@ N.hat.posbernoulli <-
   zeddij <- cbind(0, t(apply(y, 1, cumsum)))  # tau + 1 columns
   zij <- (0 + (zeddij > 0))[, 1:tau]  # 0 or 1.
   if (rename) {
-    colnames(zij) <- paste(name, 1:ncol(y), sep = "")
+    colnames(zij) <- param.names(name, ncol(y))
   } else {
     if (length(colnames(y)))
       colnames(zij) <- colnames(y)
@@ -163,8 +167,7 @@ N.hat.posbernoulli <-
        y0i       = cp1 - 1,
        yr0i      = tau - cp1 - yr1i,
        yr1i      = yr1i)
-}
-
+}  # aux.posbernoulli.t
 
 
 
@@ -180,7 +183,7 @@ rposbern <-
            Xmatrix = NULL,  # If is.null(Xmatrix) then it is created
            cap.effect =  1,
            is.popn = FALSE,
-           link = "logit",
+           link = "logitlink",
            earg.link = FALSE) {
 
 
@@ -219,12 +222,11 @@ rposbern <-
 
   Ymatrix <- matrix(0, use.n, nTimePts,
                     dimnames = list(as.character(1:use.n),
-                                    paste("y", 1:nTimePts, sep = "")))
+                                    param.names("y", nTimePts)))
 
   CHmatrix <- matrix(0, use.n, nTimePts,
                      dimnames = list(as.character(1:use.n),
-                                     paste("ch", 1:(nTimePts  ),
-                                           sep = "")))
+                                     param.names("ch", nTimePts)))
 
 
   if (is.null(Xmatrix)) {
@@ -234,7 +236,7 @@ rposbern <-
                        matrix(runif(n = use.n * (pvars-1)),
                               use.n, pvars - 1,
                               dimnames = list(as.character(1:use.n),
-                                         paste("x", 2:pvars, sep = ""))))
+                                         param.names("x", pvars)[-1])))
   }
 
 
@@ -299,7 +301,7 @@ rposbern <-
   attr(ans, "n")          <- n
 
   ans
-}
+}  # rposbern
 
 
 
@@ -326,188 +328,7 @@ dposbern <- function(x, prob, prob0 = prob, log = FALSE) {
   AA0 <- exp(logAA0)
   ell1 <- x * log(prob) + (1 - x) * log1p(-prob) - log1p(-AA0) / ncol(x)
   if (log.arg) ell1 else exp(ell1)
-}
-
-
-
-
-
-
-
-
-
-
-prob.munb.size.VGAM <- function(munb, size) {
-  prob <- size / (size + munb)
-  inf.munb <- is.infinite(munb)
-  inf.size <- is.infinite(size)
-  prob[inf.munb] <- 0
-  prob[inf.size] <- 1
-  prob[inf.munb & inf.size] <- NaN
-  prob[size < 0 | munb < 0] <- NaN
-  prob
-}
-
-
-
-dposnegbin <- function(x, size, prob = NULL, munb = NULL, log = FALSE) {
-  if (length(munb)) {
-    if (length(prob))
-      stop("arguments 'prob' and 'munb' both specified")
-  } else {
-    if (!length(prob))
-      stop("Only one of 'prob' or 'munb' must be specified")
-  }
-
-  if (!is.logical(log.arg <- log) || length(log) != 1)
-    stop("bad input for argument 'log'")
-  rm(log)
-
-
-  LLL <- max(length(x), length(prob), length(munb), length(size))
-  if (length(x)    != LLL) x    <- rep_len(x,    LLL)
-  if (length(size) != LLL) size <- rep_len(size, LLL)
-  ans <- if (length(munb)) {
-    if (length(munb) != LLL) munb <- rep_len(munb, LLL)
-    dnbinom(x = x, size = size, mu   = munb, log = TRUE)
-  } else {
-    if (length(prob) != LLL) prob <- rep_len(prob, LLL)
-    dnbinom(x = x, size = size, prob = prob, log = TRUE)
-  }
-
-  index0 <- (x == 0) & !is.na(size)  # & (!is.na(prob) |  !is.na(munb))
-  ans[ index0] <- log(0.0)
-  ans[!index0] <- ans[!index0] - (
-    if (length(prob))
-      pnbinom(0, size = size[!index0], prob = prob[!index0],
-              lower.tail = FALSE, log.p = TRUE) else
-      pnbinom(0, size = size[!index0], mu   = munb[!index0],
-              lower.tail = FALSE, log.p = TRUE))
-
-
-  if (!log.arg)
-    ans <- exp(ans)
-
-  if (!length(prob))
-    prob <- prob.munb.size.VGAM(munb, size)
-  ans[prob == 0 | prob == 1] <- NaN
-
-  ans
-}
-
-
-
-pposnegbin <- function(q, size, prob = NULL, munb = NULL,
-                       lower.tail = TRUE, log.p = FALSE) {
-
-  if (length(munb)) {
-    if (length(prob))
-      stop("arguments 'prob' and 'munb' both specified")
-  } else {
-    if (!length(prob))
-      stop("Only one of 'prob' or 'munb' must be specified")
-  }
-
-  LLL <- max(length(q), length(prob), length(munb), length(size))
-  if (length(q)    != LLL) q    <- rep_len(q,    LLL)
-  if (length(size) != LLL) size <- rep_len(size, LLL)
-  if (length(munb)) {
-    if (length(munb) != LLL) munb <- rep_len(munb, LLL)
-  } else {
-    if (length(prob) != LLL) prob <- rep_len(prob, LLL)
-  }
-
-  tail.prob <-
-    if (length(prob)) dnbinom(0, size = size, prob = prob) else
-                      dnbinom(0, size = size, mu   = munb)
-
-  vall <- rep_len(ifelse(lower.tail, log(0), log(1)), LLL)
-  ans <- if (length(prob)) {
-    ifelse(q < 1,
-           vall,
-           (if (lower.tail)
-           log(pnbinom(q, size = size, prob = prob) - tail.prob) else
-           pnbinom(q, size = size, prob = prob,
-                   lower.tail = FALSE, log.p = TRUE)) -
-           log1p(-tail.prob))
-  } else {
-    ifelse(q < 1,
-           vall,
-           (if (lower.tail)
-           log(pnbinom(q, size = size, mu   = munb) - tail.prob) else
-           pnbinom(q, size = size, mu   = munb,
-                       lower.tail = FALSE, log.p = TRUE)) -
-           log1p(-tail.prob))
-  }
-
-  if (!log.p)
-    ans <- exp(ans)
-
-  if (!length(prob))
-    prob <- prob.munb.size.VGAM(munb, size)
-  ans[prob == 0 | prob == 1] <- NaN
-
-  ans
-}
-
-
-
-qposnegbin <- function(p, size, prob = NULL, munb = NULL) {
-
-
-  if (length(munb)) {
-    if (length(prob))
-      stop("arguments 'prob' and 'munb' both specified")
-  } else {
-    if (!length(prob))
-      stop("Only one of 'prob' or 'munb' must be specified")
-  }
-
-  ans <- if (length(munb)) {
-    qnbinom(pnbinom(0, size = size, mu   = munb,
-                    lower.tail = FALSE) * p +
-            dnbinom(0, size = size, mu   = munb),
-            size = size, mu   = munb)
-  } else {
-    qnbinom(pnbinom(0, size = size, prob = prob,
-                    lower.tail = FALSE) * p +
-            dnbinom(0, size = size, prob = prob),
-            size = size, prob = prob)
-  }
-
-  ans[p == 1] <- Inf
-  ans[p < 0 | 1 < p] <- NaN
-
-  if (!length(prob))
-    prob <- prob.munb.size.VGAM(munb, size)
-  ans[prob == 0 | prob == 1] <- NaN
-
-  ans
-}
-
-
-
-rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
-  if (length(munb)) {
-    if (length(prob))
-      stop("arguments 'prob' and 'munb' both specified")
-  } else {
-    if (!length(prob))
-      stop("Only one of 'prob' or 'munb' must be specified")
-  }
-
-  ans <- if (length(munb)) {
-    qnbinom(runif(n, min = dnbinom(0, size = size, mu   = munb)),
-            size = size, mu   = munb)
-  } else {
-    qnbinom(runif(n, min = dnbinom(0, size = size, prob = prob)),
-            size = size, prob = prob)
-  }
-  if (!length(prob))
-    prob <- prob.munb.size.VGAM(munb, size)
-  ans[prob == 0 | prob == 1] <- NaN
-  ans
-}
+}  # dposbern
 
 
 
@@ -529,29 +350,37 @@ rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
         df02.dkmat2 <- df02.dkmat2[1]
       }
 
-      y.min <- 0  # Same as negbinomial() actually. A fixed const really
+      y.min <- 0  # Same as negbinomial(). A fixed const really
 
       if (!is.numeric(y.max)) {
         eff.p <- sort(c(cutoff.prob, 1 - cutoff.prob))
-        y.max <- max(qposnegbin(p = eff.p[2],
-                                munb = munb, size = size)) + 10
+        y.max <- max(qgaitnbinom(p = eff.p[2], truncate = 0,
+                                 size, munb.p = munb)) + 10
       }
 
-      Y.mat <- if (intercept.only) y.min:y.max else
+      Y.mat <- if (intercept.only) rbind(y.min:y.max) else
                matrix(y.min:y.max, length(munb), y.max-y.min+1,
                       byrow = TRUE)
   neff.row <- ifelse(intercept.only, 1, nrow(Y.mat))
   neff.col <- ifelse(intercept.only, length(Y.mat), ncol(Y.mat))
 
-      if (FALSE) {
+    if (TRUE) {
       Y.mat2 <- Y.mat + 1
       trigg.term0 <- if (intercept.only) {
-         dposnegbin(Y.mat2, size=size, munb=munb) %*% trigamma(Y.mat2+size)
+
+
+        sum(c(dgaitnbinom(Y.mat2, size[1], munb.p = munb[1],
+                          truncate = 0)) *
+        c(trigamma(Y.mat2 + size[1])))
       } else {
-         rowSums(dposnegbin(Y.mat2, size = size, munb = munb) *
-                 trigamma(Y.mat2 + size))
-      }
       }
+    }  # FALSE
+
+
+
+
+
+ 
 
 
   trigg.term <-
@@ -560,8 +389,8 @@ rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
       as.integer(intercept.only),
       as.double(neff.row), as.double(neff.col),
       as.double(size),
-      as.double(pposnegbin(Y.mat, size = size, munb = munb,
-                           lower.tail = FALSE)),
+      as.double(1 - pgaitnbinom(Y.mat, size, munb.p = munb, truncate = 0
+                            )),
       rowsums = double(neff.row))
       answerC$rowsums
   }
@@ -569,14 +398,14 @@ rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
 
 
       mymu <- munb / (1 - prob0)  # E(Y)
-      ned2l.dk2 <- trigg.term -
-         munb / (size * (size + munb)) - (mymu - munb) / (munb + size)^2
+      ned2l.dk2 <- trigg.term - munb / (size * (size + munb)) -
+        (mymu - munb) / (munb + size)^2
 
       if (second.deriv)
         ned2l.dk2 <- ned2l.dk2 - df02.dkmat2 / (1 - prob0) -
          (df0.dkmat / (1 - prob0))^2
       ned2l.dk2
-    }  # end of EIM.posNB.specialp()
+}  # end of EIM.posNB.specialp()
 
 
 
@@ -604,18 +433,21 @@ rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
 
       if (!is.numeric(y.max)) {
         eff.p <- sort(c(cutoff.prob, 1 - cutoff.prob))
-        y.max <- max(qposnegbin(p = eff.p[2],
-                                munb = munb, size = size)) + 10
+        y.max <- max(qgaitnbinom(p = eff.p[2], truncate = 0,
+                                 size, munb.p = munb)) + 10
       }
 
-      Y.mat <- if (intercept.only) y.min:y.max else
+      Y.mat <- if (intercept.only) rbind(y.min:y.max) else
                matrix(y.min:y.max, length(munb),
                       y.max-y.min+1, byrow = TRUE)
       trigg.term <- if (intercept.only) {
-         dposnegbin(Y.mat, size = size, munb = munb) %*%
-             trigamma(Y.mat + size)
+
+         sum(c(dgaitnbinom(Y.mat, size[1], munb.p = munb[1],
+                           truncate = 0)) *
+             c(trigamma(Y.mat + size[1])))
       } else {
-         rowSums(dposnegbin(Y.mat, size = size, munb = munb) *
+         rowSums(dgaitnbinom(Y.mat, size, munb.p = munb,
+                             truncate = 0) *
                  trigamma(Y.mat + size))
       }
 
@@ -626,7 +458,7 @@ rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
         ned2l.dk2 <- ned2l.dk2 - df02.dkmat2 / (1 - prob0) -
          (df0.dkmat / (1 - prob0))^2
       ned2l.dk2
-    }  # end of EIM.posNB.speciald()
+}  # end of EIM.posNB.speciald()
 
 
 
@@ -634,8 +466,8 @@ rposnegbin <- function(n, size, prob = NULL, munb = NULL) {
 
 posNBD.Loglikfun2 <- function(munbval, sizeval,
                               y, x, w, extraargs) {
-  sum(c(w) * dposnegbin(x = y, munb = munbval,
-                        size = sizeval, log = TRUE))
+  sum(c(w) * dgaitnbinom(y, sizeval, munb.p = munbval,
+                         truncate = 0, log = TRUE))
 }
 
 
@@ -656,7 +488,7 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
            eps.trig = 1e-7,
            max.support = 4000,  # 20160201; I have changed this
            max.chunk.MB = 30,  # max.memory = Inf is allowed
-           lmunb = "loge", lsize = "loge",
+           lmunb = "loglink", lsize = "loglink",
            imethod = 1,
            imunb = NULL,
            iprobs.y = NULL,  # 0.35,
@@ -754,9 +586,10 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
     extra$type.fitted <- .type.fitted
     extra$colnames.y  <- colnames(y)
 
-    predictors.names <- c(
-    namesof(param.names("munb", NOS), .lmunb , earg = .emunb , tag=FALSE),
-    namesof(param.names("size", NOS), .lsize , earg = .esize , tag=FALSE))
+    predictors.names <- c(namesof(param.names("munb", NOS, skip1 = TRUE),
+                                  .lmunb , earg = .emunb , tag = FALSE),
+                          namesof(param.names("size", NOS, skip1 = TRUE),
+                                  .lsize , earg = .esize , tag = FALSE))
     predictors.names <- predictors.names[interleave.VGAM(M, M1 = M1)]
 
     gprobs.y <- .gprobs.y
@@ -840,7 +673,8 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
     kmat <- eta2theta(eta[, !TF, drop = FALSE], .lsize , earg = .esize )
    small.size <- 1e-10
    if (any(ind4 <- (kmat < small.size))) {
-     warning("estimates of 'size' are very small. Taking evasive action.")
+     warning("estimates of 'size' are very small. ",
+             "Taking evasive action.")
      kmat[ind4] <- small.size
    }
 
@@ -866,8 +700,8 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
   last = eval(substitute(expression({
     temp0303 <- c(rep_len( .lmunb , NOS),
                   rep_len( .lsize , NOS))
-    names(temp0303) <- c(param.names("munb", NOS),
-                         param.names("size", NOS))
+    names(temp0303) <- c(param.names("munb", NOS, skip1 = TRUE),
+                         param.names("size", NOS, skip1 = TRUE))
     temp0303  <- temp0303[interleave.VGAM(M, M1 = M1)]
     misc$link <- temp0303  # Already named
 
@@ -900,7 +734,8 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
       stop("loglikelihood residuals not implemented yet")
     } else {
       ll.elts <-
-        c(w) * dposnegbin(x = y, size = kmat, munb = munb, log = TRUE)
+        c(w) * dgaitnbinom(y, kmat, munb.p = munb,
+                           truncate = 0, log = TRUE)
       if (summation) {
         sum(ll.elts)
       } else {
@@ -927,7 +762,7 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
                       .lmunb, earg = .emunb )
     kmat <- eta2theta(eta[, c(FALSE, TRUE), drop = FALSE],
                       .lsize, earg = .esize )
-    rposnegbin(nsim * length(munb), size = kmat, munb = munb)
+    rgaitnbinom(nsim * length(munb), kmat, munb.p = munb, truncate = 0)
   }, list( .lmunb = lmunb, .lsize = lsize,
            .emunb = emunb, .esize = esize ))),
 
@@ -1056,9 +891,8 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
     for (jay in 1:NOS) {
       eff.p <- sort(c( .cutoff.prob , 1 - .cutoff.prob ))
       Q.mins <- 1
-      Q.maxs <-      qposnegbin(p    = eff.p[2] ,
-                                munb = munb[, jay],
-                                size = kmat[, jay]) + 10
+      Q.maxs <- qgaitnbinom(p = eff.p[2], truncate = 0,
+                            kmat[, jay], munb.p = munb[, jay]) + 10
 
 
       eps.trig <- .eps.trig
@@ -1118,7 +952,8 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
         kkvec <- kmat[ii.TF, jay]
         muvec <- munb[ii.TF, jay]
         for (ii in 1:( .nsimEIM )) {
-          ysim <- rposnegbin(sum(ii.TF), munb = muvec, size = kkvec)
+        ysim <- rgaitnbinom(sum(ii.TF), kkvec, munb.p = muvec,
+                            truncate = 0)
           dl.dk <- digamma(ysim + kkvec) - digamma(kkvec) -
                    (ysim - muvec) / (muvec + kkvec) +
                    log1p(-muvec / (kkvec + muvec)) +
@@ -1144,7 +979,6 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
 
     save.weights <- !all(ind2)
 
-
     ned2l.dmunb2 <- mymu / munb^2 -
         ((1 + mymu/kmat) / kmat) / (1 + munb/kmat)^2 -
         df02.dmunb2 / oneminusf0 -
@@ -1155,7 +989,8 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
     ned2l.dmunbsize <- (munb - mymu) / (munb + kmat)^2 -
       df02.dkmat.dmunb / oneminusf0 -
       df0.dmunb * df0.dkmat / oneminusf0^2
-    wz[, M + M1*(1:NOS) - 1] <- ned2l.dmunbsize * dmunb.deta * dsize.deta
+     wz[, M + M1*(1:NOS) - 1] <- ned2l.dmunbsize *
+                                 dmunb.deta * dsize.deta
 
 
 
@@ -1166,7 +1001,7 @@ posnegbinomial.control <- function(save.weights = TRUE, ...) {
             .max.chunk.MB = max.chunk.MB,
             .nsimEIM = nsimEIM ))))
 
-}
+}  # posnegbinomial
 
 
 
@@ -1225,76 +1060,12 @@ rposgeom <- function(n, prob) {
 
 
 
-
-
-
-
-dpospois <- function(x, lambda, log = FALSE) {
-  if (!is.logical(log.arg <- log) || length(log) != 1)
-    stop("bad input for argument 'log'")
-  rm(log)
-
-
-  L <- max(length(x), length(lambda))
-  if (length(x)      != L) x      <- rep_len(x,      L)
-  if (length(lambda) != L) lambda <- rep_len(lambda, L)
-
-  ans <- if (log.arg) {
-    ifelse(x == 0, log(0.0), dpois(x, lambda, log = TRUE) -
-           log1p(-exp(-lambda)))
-  } else {
-    ifelse(x == 0, 0, -dpois(x, lambda) / expm1(-lambda))
-  }
-  ans[lambda <= 0] <- NaN
-  ans
-}
-
-
-
-ppospois <- function(q, lambda) {
-  L <- max(length(q), length(lambda))
-  if (length(q)      != L) q      <- rep_len(q,      L)
-  if (length(lambda) != L) lambda <- rep_len(lambda, L)
-
-  ans <- ifelse(q < 1, 0, (ppois(q, lambda) - dpois(0, lambda))
-                         / ppois(0, lambda, lower.tail = FALSE))
-  ans[lambda <= 0] <- NaN
-  ans
-}
-
-
-
-qpospois <- function(p, lambda) {
-
-
-  ans <- qpois(ppois(0, lambda, lower.tail = FALSE) * p +
-               dpois(0, lambda),
-               lambda = lambda)
-
-  ans[p == 1] <- Inf
-
-  ans[p < 0] <- NaN
-  ans[1 < p] <- NaN
-  ans[lambda <= 0] <- NaN
-  ans
-}
-
-
-
-
-rpospois <- function(n, lambda) {
-  ans <- qpois(p = runif(n, min = dpois(0, lambda)), lambda)
-  ans[lambda <= 0] <- NaN
-  ans
-}
-
-
-
-
- pospoisson <- function(link = "loge",
-                        type.fitted = c("mean", "lambda", "prob0"),
-                        expected = TRUE,
-                        ilambda = NULL, imethod = 1, zero = NULL) {
+ pospoisson <-
+  function(link = "loglink",
+           type.fitted = c("mean", "lambda", "prob0"),
+           expected = TRUE,
+           ilambda = NULL, imethod = 1, zero = NULL,
+           gt.1 = FALSE) {
 
   link <- as.list(substitute(link))
   earg <- link2list(link)
@@ -1357,26 +1128,27 @@ rpospois <- function(n, lambda) {
     extra$type.fitted <- .type.fitted
     extra$colnames.y  <- colnames(y)
 
-    mynames1 <- param.names("lambda", ncoly)
+    mynames1 <- param.names("lambda", ncoly, skip1 = TRUE)
     predictors.names <- namesof(mynames1, .link , earg = .earg,
                                 tag = FALSE)
 
     if (!length(etastart)) {
-      lambda.init <- Init.mu(y = y, w = w, imethod = .imethod ,
-                             imu = .ilambda )
-
-      etastart <- theta2eta(lambda.init, .link , earg = .earg)
+      lambda.init <- if (length( .ilambda ))
+        rep( .ilambda , length = n) else
+        Init.mu(y = y, w = w, imethod = .imethod ,
+               imu = .ilambda )
+      etastart <- theta2eta(lambda.init, .link , earg = .earg )
     }
   }), list( .link = link, .earg = earg,
             .ilambda = ilambda, .imethod = imethod,
             .type.fitted = type.fitted ))),
   linkinv = eval(substitute(function(eta, extra = NULL) {
     NOS <- NCOL(eta) / c(M1 = 1)
-   type.fitted <- if (length(extra$type.fitted)) extra$type.fitted else {
-                     warning("cannot find 'type.fitted'. ",
-                             "Returning the 'mean'.")
-                     "mean"
-                   }
+   type.fitted <- if (length(extra$type.fitted))
+     extra$type.fitted else {
+     warning("cannot find 'type.fitted'. Returning the 'mean'.")
+     "mean"
+   }
 
     type.fitted <- match.arg(type.fitted,
                      c("mean", "lambda", "prob0"))[1]
@@ -1403,13 +1175,12 @@ rpospois <- function(n, lambda) {
   }), list( .link = link, .earg = earg, .expected = expected ))),
   loglikelihood = eval(substitute(
     function(mu, y, w, residuals = FALSE, eta,
-             extra = NULL,
-             summation = TRUE) {
+             extra = NULL, summation = TRUE) {
     lambda <- eta2theta(eta, .link , earg = .earg )
     if (residuals) {
       stop("loglikelihood residuals not implemented yet")
     } else {
-      ll.elts <- c(w) * dpospois(x = y, lambda = lambda, log = TRUE)
+      ll.elts <- c(w) * dgaitpois(y, lambda, truncate = 0, log = TRUE)
       if (summation) {
         sum(ll.elts)
       } else {
@@ -1420,9 +1191,11 @@ rpospois <- function(n, lambda) {
   vfamily = c("pospoisson"),
   validparams = eval(substitute(function(eta, y, extra = NULL) {
     lambda <- eta2theta(eta, .link , earg = .earg )
-    okay1 <- all(is.finite(lambda)) && all(0 < lambda)
+    lower.bound <- if ( .gt.1 ) 1 else 0
+    okay1 <- all(is.finite(lambda)) &&
+             all(lower.bound < lambda)
     okay1
-  }, list( .link = link, .earg = earg ))),
+  }, list( .link = link, .earg = earg, .gt.1 = gt.1 ))),
 
 
   simslot = eval(substitute(
@@ -1434,7 +1207,7 @@ rpospois <- function(n, lambda) {
       warning("ignoring prior weights")
     eta <- predict(object)
     lambda <- eta2theta(eta, .link , earg = .earg )
-    rpospois(nsim * length(lambda), lambda)
+    rgaitpois(nsim * length(lambda), lambda, truncate = 0)
   }, list( .link = link, .earg = earg ))),
 
 
@@ -1447,7 +1220,6 @@ rpospois <- function(n, lambda) {
     dl.dlambda <- y / lambda - 1 - 1 / temp6
 
     dlambda.deta <- dtheta.deta(lambda, .link , earg = .earg )
-
     c(w) * dl.dlambda * dlambda.deta
   }), list( .link = link, .earg = earg ))),
   weight = eval(substitute(expression({
@@ -1457,11 +1229,12 @@ rpospois <- function(n, lambda) {
     } else {
       d2l.dlambda2 <- y / lambda^2 - (1 + 1 / temp6 + 1) / temp6
       d2lambda.deta2 <- d2theta.deta2(lambda, .link , earg = .earg)
-      wz <- (dlambda.deta^2) * d2l.dlambda2 - dl.dlambda * d2lambda.deta2
+      wz <- (dlambda.deta^2) * d2l.dlambda2 -
+            dl.dlambda * d2lambda.deta2
     }
     c(w) * wz
   }), list( .link = link, .earg = earg, .expected = expected ))))
-}
+}  # pospoisson
 
 
 
@@ -1470,85 +1243,8 @@ rpospois <- function(n, lambda) {
 
 
 
-
-dposbinom <- function(x, size, prob, log = FALSE) {
-  if (!is.logical(log.arg <- log) || length(log) != 1)
-    stop("bad input for argument 'log'")
-  rm(log)
-
-
-  L <- max(length(x), length(size), length(prob))
-  if (length(x)      != L) x    <- rep_len(x,    L)
-  if (length(size)   != L) size <- rep_len(size, L)
-  if (length(prob)   != L) prob <- rep_len(prob, L)
-
-  answer <- NaN * x
-  is0 <- (x == 0)
-  ok2 <- (prob > 0) & (prob <= 1) &
-         (size == round(size)) & (size > 0)
-
-  answer <-        dbinom(x = x, size = size, prob = prob, log = TRUE) -
-            log1p(-dbinom(x = 0, size = size, prob = prob))
-  answer[!ok2] <- NaN
-  if (log.arg) {
-    answer[is0 & ok2] <- log(0.0)
-  } else {
-    answer <- exp(answer)
-    answer[is0 & ok2] <- 0.0
-  }
-  answer
-}
-
-
-
-pposbinom <- function(q, size, prob
-                     ) {
-
-
-  if (!is.Numeric(prob, positive = TRUE))
-    stop("no zero or non-numeric values allowed for argument 'prob'")
-  L <- max(length(q), length(size), length(prob))
-  if (length(q)      != L) q      <- rep_len(q,      L)
-  if (length(size)   != L) size   <- rep_len(size,   L)
-  if (length(prob)   != L) prob   <- rep_len(prob,   L)
-
-  ifelse(q < 1, 0,
-        (pbinom(q = q, size, prob) - dbinom(x = 0, size, prob))
-       / pbinom(q = 0, size, prob, lower.tail = FALSE))
-}
-
-
-qposbinom <- function(p, size, prob
-                     ) {
-
-
-
-  ans <- qbinom(pbinom(0, size, prob, lower.tail = FALSE) * p +
-                dbinom(0, size, prob),
-                size, prob)
-
-  ans[p == 1] <- size[p == 1]
-
-  ans[p == 0] <- 1
-  ans[prob == 0] <- NaN
-
-  ans[p < 0] <- NaN
-  ans[1 < p] <- NaN
-  ans
-}
-
-
-
-rposbinom <- function(n, size, prob) {
-  qbinom(p = runif(n, min = dbinom(0, size, prob)), size, prob)
-}
-
-
-
-
-
  posbinomial <-
-  function(link = "logit",
+  function(link = "logitlink",
            multiple.responses = FALSE, parallel = FALSE,
            omit.constant = FALSE,
 
@@ -1651,7 +1347,7 @@ rposbinom <- function(n, size, prob) {
       dn2 <- if (length(dn2)) {
         paste("E[", dn2, "]", sep = "")
       } else {
-        paste("prob", 1:M, sep = "")
+        param.names("prob", M)
       }
       predictors.names <-
         namesof(if (M > 1) dn2 else "prob",
@@ -1702,7 +1398,6 @@ rposbinom <- function(n, size, prob) {
   list( .link = link, .earg = earg,
         .multiple.responses = multiple.responses ))),
   last = eval(substitute(expression({
-    extra$w <- NULL  # Kill it off
 
 
     misc$link <- rep_len( .link , M)
@@ -1726,11 +1421,11 @@ rposbinom <- function(n, size, prob) {
     if (length(extra$tau)) {
       R <- tfit$qr$qr[1:ncol.X.vlm, 1:ncol.X.vlm, drop = FALSE]
       R[lower.tri(R)] <- 0
-      tmp6 <- N.hat.posbernoulli(eta = eta, link = .link , earg = .earg ,
-                                 R = R, w = w,
-                                 X.vlm = X.vlm.save,
-                                 Hlist = Hlist,  # 20150428; bug fixed here
-                                 extra = extra, model.type = "0")
+      tmp6 <- N.hat.posbernoulli(eta = eta, link = .link ,
+                         earg = .earg , R = R, w = w,
+                         X.vlm = X.vlm.save,
+                         Hlist = Hlist,  # 20150428; bug fixed here
+                         extra = extra, model.type = "0")
       extra$N.hat    <- tmp6$N.hat
       extra$SE.N.hat <- tmp6$SE.N.hat
     }
@@ -1748,14 +1443,15 @@ rposbinom <- function(n, size, prob) {
       ycounts <- if ( .multiple.responses ) {
                   round(y * extra$orig.w)
                  } else {
-                   if (is.numeric(extra$orig.w)) y * w / extra$orig.w else
-                   y * w  # Convert proportions to counts
+                   if (is.numeric(extra$orig.w))
+                     y * w / extra$orig.w else
+                     y * w  # Convert proportions to counts
                  }
       nvec <- if ( .multiple.responses ) {
                 w
               } else {
-                if (is.numeric(extra$orig.w)) round(w / extra$orig.w) else
-                  round(w)
+                if (is.numeric(extra$orig.w))
+                  round(w / extra$orig.w) else round(w)
               }
       use.orig.w <- if (is.numeric(extra$orig.w)) extra$orig.w else 1
     binprob <- eta2theta(eta, .link , earg = .earg )
@@ -1763,10 +1459,10 @@ rposbinom <- function(n, size, prob) {
     if (residuals) {
       stop("loglikelihood residuals not implemented yet")
     } else {
-      answer <- c(use.orig.w) * dposbinom(x = ycounts, size = nvec,
-                                          prob = binprob, log = TRUE)
+      answer <- c(use.orig.w) *
+        dgaitbinom(ycounts, nvec, binprob, truncate = 0, log = TRUE)
       if ( .omit.constant ) {
-        answer <- answer - c(use.orig.w) * lchoose(n = nvec, k = ycounts)
+        answer <- answer - c(use.orig.w) * lchoose(nvec, ycounts)
       }
       ll.elts <- answer
       if (summation) {
@@ -1813,7 +1509,7 @@ rposbinom <- function(n, size, prob) {
               if (is.numeric(extra$orig.w)) round(w / extra$orig.w) else
                 round(w)
             }
-    rposbinom(nsim * length(eta), size = nvec, prob = binprob)
+    rgaitbinom(nsim * length(eta), nvec, binprob, truncate = 0)
   }, list( .link = link, .earg = earg,
           .multiple.responses = multiple.responses,
           .omit.constant = omit.constant ))),
@@ -1858,8 +1554,7 @@ rposbinom <- function(n, size, prob) {
     wz
   }), list( .link = link, .earg = earg,
             .multiple.responses = multiple.responses ))))
-}
-
+}  # posbinomial
 
 
 
@@ -1867,7 +1562,7 @@ rposbinom <- function(n, size, prob) {
 
 
  posbernoulli.t <-
-  function(link = "logit",
+  function(link = "logitlink",
 
            parallel.t = FALSE ~ 1,
 
@@ -1875,13 +1570,17 @@ rposbinom <- function(n, size, prob) {
 
            iprob = NULL,
 
-           p.small = 1e-4, no.warning = FALSE) {
+           p.small = 1e-4, no.warning = FALSE,
+           type.fitted = c("probs", "onempall0")) {
 
 
 
 
 
 
+  type.fitted <- match.arg(type.fitted,
+                           c("probs", "onempall0"))[1]
+
 
   apply.parint <- FALSE
 
@@ -1929,12 +1628,14 @@ rposbinom <- function(n, size, prob) {
          multipleResponses = TRUE,
          parameters.names = c("prob"),
          p.small    = .p.small ,
+         type.fitted  = .type.fitted ,
          no.warning = .no.warning ,
          apply.parint = .apply.parint ,
          parallel.t = .parallel.t )
   }, list( .parallel.t   = parallel.t,
-           .p.small    = p.small,
-           .no.warning = no.warning,
+           .p.small      = p.small,
+           .no.warning   = no.warning,
+           .type.fitted  = type.fitted,
            .apply.parint = apply.parint ))),
 
   initialize = eval(substitute(expression({
@@ -1943,13 +1644,15 @@ rposbinom <- function(n, size, prob) {
     mustart.orig <- mustart
     y <- as.matrix(y)
     M <- ncoly <- ncol(y)
-    extra$ncoly       <- ncoly <- ncol(y)
+    extra$ncoly      <- ncoly <- ncol(y)
     extra$tau <- tau <- ncol(y)
     extra$orig.w <- w
 
     extra$p.small    <- .p.small
     extra$no.warning <- .no.warning
 
+    extra$type.fitted <- .type.fitted
+    extra$colnames.y  <- colnames(y)
 
     w <- matrix(w, n, ncoly)
     mustart <- matrix(colSums(y) / colSums(w),
@@ -1976,7 +1679,7 @@ rposbinom <- function(n, size, prob) {
     dn2 <- if (length(dn2)) {
       paste("E[", dn2, "]", sep = "")
     } else {
-      paste("prob", 1:M, sep = "")
+      param.names("prob", M)
     }
 
 
@@ -1995,10 +1698,20 @@ rposbinom <- function(n, size, prob) {
     }
     mustart <- NULL
   }), list( .link = link, .earg = earg,
-            .p.small    = p.small,
-            .no.warning = no.warning
+            .p.small      = p.small,
+            .type.fitted  = type.fitted,
+            .no.warning   = no.warning
            ))),
   linkinv = eval(substitute(function(eta, extra = NULL) {
+    type.fitted <-
+      if (length(extra$type.fitted)) extra$type.fitted else {
+        warning("cannot find 'type.fitted'. Returning the 'probs'.")
+        "probs"
+      }
+
+    type.fitted <- match.arg(type.fitted,
+                     c("probs", "onempall0"))[1]
+
     tau <- extra$ncoly
     probs <- eta2theta(eta, .link , earg = .earg )
     logAA0 <- rowSums(log1p(-probs))
@@ -2008,8 +1721,11 @@ rposbinom <- function(n, size, prob) {
 
 
     fv <- probs / AAA
-    fv
-  }, list( .link = link, .earg = earg ))),
+   ans <- switch(type.fitted,
+                  "probs"      = fv,
+                  "onempall0"  = AAA)
+    label.cols.y(ans, colnames.y = extra$colnames.y, NOS = NOS)
+}, list( .link = link, .earg = earg ))),
   last = eval(substitute(expression({
     extra$w   <- NULL   # Kill it off
 
@@ -2031,7 +1747,7 @@ rposbinom <- function(n, size, prob) {
     tmp6 <- N.hat.posbernoulli(eta = eta, link = .link , earg = .earg ,
                                R = R, w = w,
                                X.vlm = X.vlm.save,
-                               Hlist = Hlist,  # 20150428; bug fixed here
+                               Hlist = Hlist,  # 20150428 bug fixed here
                                extra = extra, model.type = "t")
     extra$N.hat    <- tmp6$N.hat
     extra$SE.N.hat <- tmp6$SE.N.hat
@@ -2110,17 +1826,13 @@ rposbinom <- function(n, size, prob) {
 
     for (slocal in 1:(M-1))
       for (tlocal in (slocal+1):M)
-        wz[, iam(slocal, tlocal, M = M)] <- dprobs.deta[, slocal] *
-                                            dprobs.deta[, tlocal] *
-                                            (B.st[, slocal,tlocal] +
-                                             B.s [, slocal] *
-                                             B.s [, tlocal] / AAA) / (-AAA)
-
-
-
+        wz[, iam(slocal, tlocal, M = M)] <-
+          dprobs.deta[, slocal] * dprobs.deta[, tlocal] *
+          (B.st[, slocal, tlocal] +
+           B.s [, slocal] * B.s [, tlocal] / AAA) / (-AAA)
     wz
   }), list( .link = link, .earg = earg ))))
-}
+}  # posbernoulli.t
 
 
 
@@ -2128,7 +1840,7 @@ rposbinom <- function(n, size, prob) {
 
 
  posbernoulli.b <-
-  function(link = "logit",
+  function(link = "logitlink",
 
 
            drop.b = FALSE ~ 1,
@@ -2139,8 +1851,7 @@ rposbinom <- function(n, size, prob) {
            I2 = FALSE,
            ipcapture = NULL,
            iprecapture = NULL,
-           p.small = 1e-4, no.warning = FALSE
-           ) {
+           p.small = 1e-4, no.warning = FALSE) {
 
 
 
@@ -2491,14 +2202,15 @@ rposbinom <- function(n, size, prob) {
     wz <- c(w) * wz
     wz
   }), list( .link = link, .earg = earg ))))
-}
+}  # posbernoulli.b
+
 
 
 
 
 
  posbernoulli.tb <-
-  function(link = "logit",
+  function(link = "logitlink",
            parallel.t = FALSE ~  1,
            parallel.b = FALSE ~  0,
            drop.b     = FALSE ~  1,
@@ -2506,7 +2218,7 @@ rposbinom <- function(n, size, prob) {
            imethod = 1,
            iprob = NULL,
            p.small = 1e-4, no.warning = FALSE,
-           ridge.constant = 0.01,
+           ridge.constant = 0.0001,  # 20181020
            ridge.power = -4) {
 
 
@@ -2684,8 +2396,8 @@ rposbinom <- function(n, size, prob) {
     cap.hist1  <- extra$cap.hist1  <- tmp3$cap.hist1
 
 
-    dn2.cap   <- paste("pcapture.",   1:ncoly, sep = "")
-    dn2.recap <- paste("precapture.", 2:ncoly, sep = "")
+    dn2.cap   <- param.names("pcapture.",   ncoly)
+    dn2.recap <- param.names("precapture.", ncoly)[-1]
 
     predictors.names <- c(
       namesof(dn2.cap,   .link , earg = .earg, short = TRUE),
@@ -2848,7 +2560,7 @@ rposbinom <- function(n, size, prob) {
   vfamily = c("posbernoulli.tb"),
   validparams = eval(substitute(function(eta, y, extra = NULL) {
     probs <- eta2theta(eta, .link , earg = .earg )
-    okay1 <- all(is.finite(probs)) &&  all(0 < probs & probs < 1)
+    okay1 <- all(is.finite(probs)) && all(0 < probs & probs < 1)
     okay1
   }, list( .link = link, .earg = earg ))),
 
@@ -2899,7 +2611,7 @@ rposbinom <- function(n, size, prob) {
 
     QQQcummat <- exp(t( apply(log1p(-prc), 1, cumsum)))
     wz.pc <- (QQQcummat / prc - QQQ / (1 - QQQ)) / ((1 - QQQ) *
-              (1 - prc)^2)
+             (1 - prc)^2)
     wz[, 1:tau] <- wz.pc
 
 
@@ -2924,16 +2636,26 @@ rposbinom <- function(n, size, prob) {
                dprobs.deta[, cindex$col.index]
 
 
+   if (TRUE) {  # ------------------------------------
+      wz.adjustment <- .ridge.constant * iter^( .ridge.power )
+      wz[, 1:tau] <- wz[, 1:tau] * (1 + wz.adjustment)
+   } else {  # ------------------------------------
       wz.mean <- mean(wz[, 1:tau])
       wz.adjustment <- wz.mean * .ridge.constant * iter^( .ridge.power )
       wz[, 1:tau] <- wz[, 1:tau] + wz.adjustment
+   }  # ------------------------------------
+
+
+
+
+
+
 
     c(w) * wz
   }), list( .link = link, .earg = earg,
             .ridge.constant = ridge.constant,
             .ridge.power = ridge.power ))))
-}
-
+}  # posbernoulli.tb
 
 
 
@@ -2949,51 +2671,40 @@ setClass("posbernoulli.b",      contains = "posbernoulli.tb")
 
 
 setMethod("summaryvglmS4VGAM",  signature(VGAMff = "posbernoulli.tb"),
-  function(object,
-           VGAMff,
-           ...) {
+  function(object, VGAMff, ...) {
   object@post
 })
 
 
 
-setMethod("showsummaryvglmS4VGAM",  signature(VGAMff = "posbernoulli.tb"),
-  function(object,
-           VGAMff,
-           ...) {
+setMethod("showsummaryvglmS4VGAM", signature(VGAMff = "posbernoulli.tb"),
+  function(object, VGAMff, ...) {
  if (length(object@extra$N.hat) == 1 &&
       is.numeric(object@extra$N.hat)) {
-    cat("\nEstimate of N: ", round(object@extra$N.hat, digits = 3), "\n")
-    cat("\nStd. Error of N: ", round(object@extra$SE.N.hat, digits = 3),
-        "\n")
+   cat("\nEstimate of N: ", round(object@extra$N.hat, digits = 3), "\n")
+   cat("\nStd. Error of N: ", round(object@extra$SE.N.hat, digits = 3),
+       "\n")
 
-    confint.N <- object@extra$N.hat +
-        c(Lower = -1, Upper = 1) * qnorm(0.975) * object@extra$SE.N.hat
-    cat("\nApproximate 95 percent confidence interval for N:\n")
-    print(round(confint.N, digits = 2))
+   confint.N <- object@extra$N.hat +
+       c(Lower = -1, Upper = 1) * qnorm(0.975) * object@extra$SE.N.hat
+   cat("\nApproximate 95 percent confidence interval for N:\n")
   }
 })
 
 
 
-setMethod("showsummaryvglmS4VGAM",  signature(VGAMff = "posbernoulli.b"),
-  function(object,
-           VGAMff,
-           ...) {
+setMethod("showsummaryvglmS4VGAM", signature(VGAMff = "posbernoulli.b"),
+  function(object, VGAMff, ...) {
   callNextMethod(VGAMff = VGAMff, object = object, ...)
 })
 
 
 
-setMethod("showsummaryvglmS4VGAM",  signature(VGAMff = "posbernoulli.t"),
-  function(object,
-           VGAMff,
-           ...) {
+setMethod("showsummaryvglmS4VGAM", signature(VGAMff = "posbernoulli.t"),
+  function(object, VGAMff, ...) {
   callNextMethod(VGAMff = VGAMff, object = object, ...)
 })
 
 
 
 
-
-
