@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2019 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2021 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -24,10 +24,10 @@ lrt.stat.vlm <-
 
   wald.stat.vlm(object, values0 = values0,
                 subset = subset, omit1s = omit1s, all.out = all.out,
-                iterate = TRUE,
+                iterate.SE = TRUE,
                 trace = trace,
-                as.summary = FALSE,  # Does not make sense if TRUE
-                lrt.really = TRUE, ...)
+                orig.SE = FALSE,  # Does not make sense if TRUE
+                LR.really = TRUE, ...)
 }  # lrt.stat.vlm
 
 
